unit Unit1;

// - VCC -
// Main window of JTAG CDC Loader
// For Spartan-3(E) FPGAs
// - 2010 -

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    tmrStartup: TTimer;
    ListBox1: TListBox;
    Button3: TButton;
    ProgressBar1: TProgressBar;
    Panel1: TPanel;
    LabeledEdit1: TLabeledEdit;
    Panel3: TPanel;
    ComboBox1: TComboBox;
    StatusBar1: TStatusBar;
    Button4: TButton;
    OpenDialog1: TOpenDialog;
    procedure Button1Click(Sender: TObject);
    procedure tmrStartupTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ComboBox1Change(Sender: TObject);
    procedure Button4Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function peek: Boolean; {Peek Message}
    procedure GetJTAGChain;
    procedure ProgramFPGA(ATibFile: TFileName; PositionInChain: Byte);
    procedure LoadSettingsFromIni;
    procedure SaveSettingsToIni;
  end;

const
  timeout = 2000;


  CMaxDevices = 30; {Maximum number of devices}
  CDBDeviceCount = 6; {Number of items in Device database}

  CDeviceNameArray: array[0..CDBDeviceCount] of string[9] =
    ('XC3S100E',     'XC3S500E',     'XCF02S',      'XCF04S',      'XC3S400',      'XC2C64A',      'Unknown');


var
  Form1: TForm1;
  ChainLength: Byte;
  ReceiveBuffer: array[0..63] of Char;
  AbortProgrammging: Boolean;

  COMnumber: Byte = 1;
  CurrentFileName: TFileName;

implementation

{$R *.dfm}

uses
  JTAGdataProvider, BitToTibConverter, IniFiles, WinCom;

function TForm1.peek: Boolean;
var
  msg: TMsg;
begin
  Result := False;
  if (PeekMessage(msg, 0, 0, 0, PM_REMOVE)) then
  begin
    if (msg.message = WM_QUIT) then
      Result := True
    else
    begin
      TranslateMessage(msg);
      DispatchMessage(msg);
    end
  end;
end;

procedure TForm1.GetJTAGChain;
var
  ReadResult: Cardinal;
  i: Byte;
begin
  SendString(COMnumber, #10);
  repeat
    peek;
    ReadResult := ReceiveChars(COMnumber, ReceiveBuffer, SizeOf(ReceiveBuffer));
  until ReadResult > 0;
  
  ChainLength := Ord(ReceiveBuffer[0]);
  ListBox1.Items.Add('ChainLength = ' + IntToStr(ChainLength));
  ListBox1.Items.Add('');

  for i := 1 to ChainLength do
    ListBox1.Items.Add(CDeviceNameArray[Ord(ReceiveBuffer[i])] + '  Position: ' + IntToStr(i));

  FlushReceiveBuffer(COMnumber);  
end;

procedure TForm1.ProgramFPGA(ATibFile: TFileName; PositionInChain: Byte);
var
  NoMoreData: Boolean;
  arr: ArrayOf61Chars;
  NumberOfWrittenBytes: Cardinal;
  DataLength: Byte;
  ReadResult: Cardinal;
begin
  OpenTheTibFile(ATibFile);

  SendString(COMnumber, #20 + Chr(PositionInChain)); {program command + first device}
  Sleep(1);

  ProgressBar1.Position := 0;
  AbortProgrammging := False;
  repeat

    repeat
      peek;
      ReadResult := ReceiveChars(COMnumber, ReceiveBuffer, SizeOf(ReceiveBuffer));
      if ReadResult > 0 then
        if ReceiveBuffer[0] = #25 then   {PIC asks for more data}
        begin
          FlushReceiveBuffer(COMnumber);
          Break;
        end;
    until AbortProgrammging;
    
    ReadFromTibFileAsAStream(@arr);
    DataLength := arr[0];

    if AbortProgrammging then
      arr[0] := arr[0] or 64;

      
    WriteFile(ComData[COMnumber]^.Handle, arr, arr[0] and $3F + 1, NumberOfWrittenBytes, nil);


    NoMoreData := arr[0] and 128 = 128;

    DataLength := DataLength and $7F;

    ProgressBar1.Position := ProgressBar1.Position + DataLength;
    if Random(4) = 1 then  {do not update every time}
      Panel3.Caption := 'Program pos: ' + IntToStr(ProgressBar1.Position) + ' / ' + IntToStr(ProgressBar1.Max);

  until NoMoreData or AbortProgrammging;

  CloseTheTibFile;

  Panel3.Caption := 'Program pos: ' + IntToStr(ProgressBar1.Position) + ' / ' + IntToStr(ProgressBar1.Max);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  if not ComIsOpen(COMnumber) then
    Exit;

  ListBox1.Clear;
  GetJTAGChain;
  
  if ChainLength > 0 then
    Button2.Enabled := True;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  TibFileName: string;
  PositionToProgram: Byte;
  BitLength: Integer;
begin
  if not ComIsOpen(COMnumber) then
    Exit;

  PositionToProgram := StrToInt(LabeledEdit1.Text);

  if not FileExists(CurrentFileName) then
    if not OpenDialog1.Execute then
      Exit
    else
    begin
      CurrentFileName := OpenDialog1.FileName;
      StatusBar1.Panels.Items[1].Text := CurrentFileName;
    end;

  Panel1.Caption := 'Programming';
  Panel1.Repaint;

  Button1.Enabled := False;
  Button2.Enabled := False;


  try
    if PositionToProgram > ChainLength then
      MessageBox(Handle, PChar('Position selected exceeds chain length.' + #13#10 + 'Valid positions are in range 1..' + IntToStr(ChainLength)), 'JTAG error', MB_ICONERROR)
    else
    begin
      TibFileName := ConvertBitFileToTibFile(CurrentFileName, BitLength);
      ProgressBar1.Max := BitLength;
      ProgramFPGA(TibFileName, PositionToProgram);  {Send the .tib file to microcontroller}
    end;
  finally
    Button1.Enabled := True;
    Button2.Enabled := True;
  end;

  Panel1.Caption := 'Finished';
  Panel1.Repaint;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  AbortProgrammging := True;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  if not OpenDialog1.Execute then
    Exit;
    
  CurrentFileName := OpenDialog1.FileName;
  StatusBar1.Panels.Items[1].Text := CurrentFileName;
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
  try
    CloseCom(COMnumber);
    COMnumber := ComboBox1.ItemIndex + 1;
    OpenCom(COMnumber, BaudRate_115K, Parity_none, 8, Stopbits_1, 61, 60);

    if not ComIsOpen(COMnumber) then
      StatusBar1.Panels.Items[0].Text := 'Can''t open COM port ' + IntToStr(COMnumber)
    else
      StatusBar1.Panels.Items[0].Text := 'COM port ' + IntToStr(COMnumber) + ' was open successfully';
  except

  end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  try
    CloseCom(COMnumber);
    SaveSettingsToIni;
  except
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  tmrStartup.Enabled := True;
end;

procedure TForm1.tmrStartupTimer(Sender: TObject);
begin
  tmrStartup.Enabled := False;

  if not ComIsOpen(COMnumber) then
  begin
    CurrentFileName := ExtractFilePath(ParamStr(0)) + 'lcd.bit';
    LoadSettingsFromIni;
    ComboBox1.ItemIndex := COMnumber - 1;
  end;

  CloseCom(COMnumber);

  StatusBar1.Panels.Items[0].Text := 'Opening COM port ' + IntToStr(COMnumber);
  OpenCom(COMnumber, BaudRate_115K, Parity_none, 8, Stopbits_1, 61, 60);

  if not ComIsOpen(COMnumber) then
  begin
    tmrStartup.Enabled := True;
    tmrStartup.Interval := 1000;
    StatusBar1.Panels.Items[0].Text := 'Can''t open COM port ' + IntToStr(COMnumber);
  end
  else
    StatusBar1.Panels.Items[0].Text := 'COM port ' + IntToStr(COMnumber) + ' was open successfully';
end;

procedure TForm1.LoadSettingsFromIni;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'JTAGCDCLoader.ini');
  try
    COMnumber := IniFile.ReadInteger('COM port', 'Number', 1);
    CurrentFileName := IniFile.ReadString('Bit file', 'FileName', 'lcd.bit');
    StatusBar1.Panels.Items[1].Text := CurrentFileName;

    Left := IniFile.ReadInteger('Window', 'Left', Screen.Width div 2 - Width div 2);
    Top := IniFile.ReadInteger('Window', 'Top', Screen.Height div 2 - Height div 2);
    Width := IniFile.ReadInteger('Window', 'Width', Width);
    Height := IniFile.ReadInteger('Window', 'Height', Height);

  finally

  end;
end;

procedure TForm1.SaveSettingsToIni;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'JTAGCDCLoader.ini');
  try
    IniFile.WriteInteger('COM port', 'Number', COMnumber);
    IniFile.WriteString('Bit file', 'FileName', CurrentFileName);

    IniFile.WriteInteger('Window', 'Left', Left);
    IniFile.WriteInteger('Window', 'Top', Top);
    IniFile.WriteInteger('Window', 'Width', Width);
    IniFile.WriteInteger('Window', 'Height', Height);
  finally

  end;
end;

end.
